/*
 * stm32f4_NokiaN958GB_LDS285.c
 *
 *  Created on:  10.12.2013
 *  more informations:  http://mikrocontrollerprojekte.blogspot.de/
 *
 *    Hardware:  STM32F4 Discovery (STM32F407VGT6) @ 168 MHz
 *
 *       HowTo:	 1. call LDS285_Init_Display();
 *  		     2. call LDS285_Demo();
 *
 *
 */

#include "stm32f4_NokiaN958GB_LDS285.h"


void LDS285_Init_GPIO(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	// Enable Clock Port D and Port E
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD,ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE,ENABLE);

	// InitStructur for Port D
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4  | GPIO_Pin_5 |
								  GPIO_Pin_7 |GPIO_Pin_11 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO_InitStructure);

	// alternate function Port D
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource0,  GPIO_AF_FSMC); 	// PD0 = FSMC_D2   -> DB2
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource1,  GPIO_AF_FSMC); 	// PD1 = FSMC_D3   -> DB3
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource4,  GPIO_AF_FSMC); 	// PD4 = FSMC_NOE  -> RD
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource5,  GPIO_AF_FSMC); 	// PD5 = FSMC_NWE  -> WR
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource7,  GPIO_AF_FSMC); 	// PD7 = FSMC_NE1  -> CS
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource11, GPIO_AF_FSMC); 	// PD11 = FSMC_RS  -> RS
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource14, GPIO_AF_FSMC); 	// PD14 = FSMC_D0  -> DB0
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource15, GPIO_AF_FSMC); 	// PD15 = FSMC_D1  -> DB1

	// InitStructur for Port E
	GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOE, &GPIO_InitStructure);

	// alternate function Port E
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource7,  GPIO_AF_FSMC); 	// PE7 = FSMC_D4   -> DB4
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource8,  GPIO_AF_FSMC); 	// PE8 = FSMC_D5   -> DB5
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource9,  GPIO_AF_FSMC); 	// PE9 = FSMC_D6   -> DB6
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource10, GPIO_AF_FSMC); 	// PE10 = FSMC_D7  -> DB7

	// Reset on Port E Pin 1
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;             	 	// PE1 = Reset
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOE, &GPIO_InitStructure);
}


void LDS285_Init_FSMC(void)
{
	FSMC_NORSRAMInitTypeDef        FSMC_NORSRAMInitStructure;
	FSMC_NORSRAMTimingInitTypeDef  FSMC_NORSRAMTimingInitStructure;

	// Clock Enable von FSMC
	RCC_AHB3PeriphClockCmd(RCC_AHB3Periph_FSMC, ENABLE);

	// InitStructur for FSMC
	FSMC_NORSRAMTimingInitStructure.FSMC_AddressSetupTime = 0;
	FSMC_NORSRAMTimingInitStructure.FSMC_AddressHoldTime = 0;
	FSMC_NORSRAMTimingInitStructure.FSMC_DataSetupTime = 6;
	FSMC_NORSRAMTimingInitStructure.FSMC_BusTurnAroundDuration = 0;
	FSMC_NORSRAMTimingInitStructure.FSMC_CLKDivision = 0;
	FSMC_NORSRAMTimingInitStructure.FSMC_DataLatency = 0;
	FSMC_NORSRAMTimingInitStructure.FSMC_AccessMode = FSMC_AccessMode_A;

	// InitStructur for Bank 1 / PSRAM 1
	FSMC_NORSRAMInitStructure.FSMC_Bank = FSMC_Bank1_NORSRAM1;
	FSMC_NORSRAMInitStructure.FSMC_DataAddressMux = FSMC_DataAddressMux_Disable;
	FSMC_NORSRAMInitStructure.FSMC_MemoryType = FSMC_MemoryType_SRAM;
	FSMC_NORSRAMInitStructure.FSMC_MemoryDataWidth = FSMC_MemoryDataWidth_16b;
	FSMC_NORSRAMInitStructure.FSMC_BurstAccessMode = FSMC_BurstAccessMode_Disable;
	FSMC_NORSRAMInitStructure.FSMC_AsynchronousWait = FSMC_AsynchronousWait_Disable;
	FSMC_NORSRAMInitStructure.FSMC_WaitSignalPolarity = FSMC_WaitSignalPolarity_Low;
	FSMC_NORSRAMInitStructure.FSMC_WrapMode = FSMC_WrapMode_Disable;
	FSMC_NORSRAMInitStructure.FSMC_WaitSignalActive = FSMC_WaitSignalActive_BeforeWaitState;
	FSMC_NORSRAMInitStructure.FSMC_WriteOperation = FSMC_WriteOperation_Enable;
	FSMC_NORSRAMInitStructure.FSMC_WaitSignal = FSMC_WaitSignal_Disable;
	FSMC_NORSRAMInitStructure.FSMC_ExtendedMode = FSMC_ExtendedMode_Disable;
	FSMC_NORSRAMInitStructure.FSMC_WriteBurst = FSMC_WriteBurst_Disable;
	FSMC_NORSRAMInitStructure.FSMC_ReadWriteTimingStruct = &FSMC_NORSRAMTimingInitStructure;
	FSMC_NORSRAMInitStructure.FSMC_WriteTimingStruct = &FSMC_NORSRAMTimingInitStructure;
	FSMC_NORSRAMInit(&FSMC_NORSRAMInitStructure);

	// Enable Bank 1
	FSMC_NORSRAMCmd(FSMC_Bank1_NORSRAM1, ENABLE);
}


void LDS285_Init_Display(void)
{
	LDS285_Init_GPIO();				// GPIO Init
	LDS285_Init_FSMC();				// FSMC Init

	Delay_ms_N95(20);

	GPIOE->BSRRL = GPIO_Pin_1;		// set Reset-Pin
	Delay_ms_N95(5);
	GPIOE->BSRRH = GPIO_Pin_1;		// clear Reset-Pin
	Delay_ms_N95(50);
	GPIOE->BSRRL = GPIO_Pin_1;		// set Reset-Pin
	Delay_ms_N95(50);

	writeCommand(SLEEP_OUT);
	Delay_ms_N95(10);
	writeCommand(DISPLAY_INVERSION_OFF);
	writeCommand(IDLE_MODE_OFF);
	writeCommand(NORMAL_DISPLAY_MODE_ON);

	writeCommand(VERTICAL_SCROLLING_DEFINITION);
	writeData(0x00);
	writeData(0x00);
	writeData(0x01);
	writeData(0x40);
	writeData(0x00);
	writeData(0x00);
	writeCommand(VERTICAL_SCROLLING_START_ADDRESS);
	writeData(0x00);
	writeData(0x00);

	writeCommand(MEMORY_ACCESS_CONTROL);
	writeData(0x60);

	writeCommand(INTERFACE_PIXEL_FORMAT);
	writeData(0x77);

	writeCommand(WRITE_DISPLAY_BRIGHTNESS);
	writeData(0x00);

	writeCommand(WRITE_CTRL_DISPLAY);
	writeData(0x24);
	writeCommand(WRITE_CONTENT_ADAPTIVE_BRIGHTNESS);
	writeData(0x01);

	Delay_ms_N95(125);
	writeCommand(DISPLAY_ON);

	setBacklight(100);
}


void LDS285_Demo(void)
{
	uint32_t i = 0;

	// Clear Screen
	clearScreen();
	Delay_ms_N95(1000);

	// Fill Screen Test
	for(i = 0; i < 10; i++)
	{
		fillScreen(GREEN);
		Delay_ms_N95(250);
		fillScreen(YELLOW);
		Delay_ms_N95(250);
		fillScreen(CYAN);
		Delay_ms_N95(250);
		fillScreen(RED);
		Delay_ms_N95(250);
		fillScreen(BLUE);
		Delay_ms_N95(250);
		fillScreen(ORANGE);
		Delay_ms_N95(250);
	}

	// Rectangle Test
	fillScreen(WHITE);
	for(i = 0; i < 230; i++)
	fillScreenPartial(i/2, i, i/2, i, i*100*4);
	Delay_ms_N95(4000);

	// Pixel Test
	fillScreen(WHITE);
	uint16_t x = 0;
	uint16_t y = 0;
	for(y = 0; y < 320; y++)
		for(x = 0; x < 240; x++)
			setPixel(x, y, x*y*2);
	Delay_ms_N95(4000);
}


void writeCommand(uint8_t command)
{
	((*(__IO u16 *) (LCD_Command)) = ((u16)command));
}


void writeData(uint8_t data)
{
	((*(__IO u16 *) (LCD_Data)) = ((u16)(data)));
}


void setBacklight(uint8_t prozent)
{
	uint8_t value = prozent;
	if(value > 100)
		value = 100;
	value = (uint8_t) value * 255 / 100;
	writeCommand(WRITE_DISPLAY_BRIGHTNESS);
	writeData(value);
}


void fillScreen(uint32_t color)
{
	uint32_t i = 0;

	writeCommand(COLUMN_ADDRESS_SET);	// x = 0 ... 239
	writeData(0x00);					// 0
	writeData(0x00);					// xstart
	writeData(0x00);					// 0
	writeData(0xEF);					// xend
	writeCommand(ROW_ADDRESS_SET);		// y = 0 ... 319
	writeData(0x00);					// ystart >> 8
	writeData(0x00);					// ystart & 0xff
	writeData(0x01);					// yend>>8
	writeData(0x3F);					// yend & 0xff
	writeCommand(MEMORY_WRITE);

	while(i < 76800)
	{
		writeData((color & 0x00FF0000) >> 16);	// R
		writeData((color & 0x0000FF00) >> 8);	// G
		writeData(color & 0x000000FF);			// B
		i++;
	}
}


void clearScreen(void)
{
	fillScreen(0xffffff);
}


void sleepDisplay(void)
{
	writeCommand(DISPLAY_OFF);
	writeCommand(SLEEP_IN);
}


void wakeDisplay(void)
{
	writeCommand(SLEEP_OUT);
	Delay_ms_N95(120);
	writeCommand(DISPLAY_ON);
}


void fillScreenPartial(uint32_t xstart, uint32_t xend, uint32_t ystart, uint32_t yend, uint32_t color)
{
	uint32_t i = 0;

	writeCommand(COLUMN_ADDRESS_SET);	// x = 0 ... 239
	writeData(0x00);					// 0
	writeData(xstart);					// xstart
	writeData(0x00);					// 0
	writeData(xend);					// xend
	writeCommand(ROW_ADDRESS_SET);		// y = 0 ... 319
	writeData(ystart >> 8);				// ystart >> 8
	writeData(ystart & 0xff);			// ystart & 0xff
	writeData(yend>>8);					// yend>>8
	writeData(yend & 0xff);				// yend & 0xff
	writeCommand(MEMORY_WRITE);

	while(i < 76800)
	{
		writeData((color & 0x00FF0000) >> 16);	// R
		writeData((color & 0x0000FF00) >> 8);	// G
		writeData(color & 0x000000FF);			// B
		i++;
	}
}


void setPixel(uint32_t x, uint32_t y, uint32_t color)
{
	writeCommand(COLUMN_ADDRESS_SET);
	writeData(0x00);
	writeData(x);
	writeData(0x00);
	writeData(x);
	writeCommand(ROW_ADDRESS_SET);
	writeData(y >> 8);
	writeData(y & 0xff);
	writeData((y)>>8);
	writeData((y) & 0xff);
	writeCommand(MEMORY_WRITE);

	writeData((color & 0x00FF0000) >> 16);	// R
	writeData((color & 0x0000FF00) >> 8);	// G
	writeData(color & 0x000000FF);			// B
}



void Delay_N95(__IO uint32_t nCount)
{
  while(nCount--)
  {
  }
}


void Delay_ms_N95(uint16_t ms)
{
	uint16_t count = 0;
	while(count < ms)
	{
		Delay_N95(0x0051EB);
		count++;
	}
}
