/*
 * 	stm32f4_NokiaN958GB_LDS285.h
 *
 *  Created on: 10.12.2013
 *	more informations: http://mikrocontrollerprojekte.blogspot.de/
 *
 *
 *
 *    license
 *    Copyright (c) 2011,2012,2013 Andrew Brown. All rights reserved.
 *    Redistribution and use in source and binary forms, with or without
 *    modification, are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, this
 *      list of conditions and the following disclaimer.
 *
 *    - Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 *    - The name of Andrew Brown may not be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 *
 *    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
 *    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *    IN NO EVENT SHALL ANDREW BROWN BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *    OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *    WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 *    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef STM32F4_NOKIAN958GB_LDS285_H_
#define STM32F4_NOKIAN958GB_LDS285_H_


/* Include */
#include "stm32f4xx_gpio.h"		// from ST Lib.
#include "stm32f4xx_fsmc.h"		// from ST Lib.


/* N95 8GB Display Commands */
#define	NOP									 0
#define	SOFTWARE_RESET						 1
#define	READ_DISPLAY_ID						 4
#define	READ_DISPLAY_STATUS					 9
#define	READ_DISPLAY_POWER_MODE				 0xA
#define	READ_DISPLAY_MADCTL					 0xb
#define	READ_DISPLAY_PIXEL_FORMAT			 0xc
#define	READ_DISPLAY_IMAGE_MODE				 0xd
#define	READ_DISPLAY_SIGNAL_MODE			 0xe
#define	READ_DISPLAY_SELF_DIAGNOSTICS 		 0xf
#define	SLEEP_IN 							 0x10
#define	SLEEP_OUT 							 0x11
#define	PARTIAL_MODE_ON 					 0x12
#define	NORMAL_DISPLAY_MODE_ON 				 0x13
#define	DISPLAY_INVERSION_OFF 				 0x20
#define	DISPLAY_INVERSION_ON 				 0x21
#define	GAMMA_SET 							 0x26
#define	DISPLAY_OFF 						 0x28
#define	DISPLAY_ON 							 0x29
#define	COLUMN_ADDRESS_SET 					 0x2a
#define	ROW_ADDRESS_SET 					 0x2b
#define	MEMORY_WRITE 						 0x2c
#define	MEMORY_READ 						 0x2e
#define	PARTIAL_AREA						 0x30
#define	VERTICAL_SCROLLING_DEFINITION 		 0x33
#define	TEARING_EFFECT_LINE_OFF 			 0x34
#define	TEARING_EFFECT_LINE_ON 				 0x35
#define	MEMORY_ACCESS_CONTROL 				 0x36
#define	VERTICAL_SCROLLING_START_ADDRESS 	 0x37
#define	IDLE_MODE_OFF 						 0x38
#define	IDLE_MODE_ON 						 0x39
#define	INTERFACE_PIXEL_FORMAT				 0x3a
#define	WRITE_DISPLAY_BRIGHTNESS			 0x51
#define	WRITE_CTRL_DISPLAY					 0x53
#define	WRITE_CONTENT_ADAPTIVE_BRIGHTNESS 	 0x55



#define LCD_Data    	((u32)0x60020000)    // Disp. Data RS = 1
#define LCD_Command    	((u32)0x60000000)    // Disp. CMD RS = 0


// Colors
#define WHITE		0xffffff
#define BLACK		0x000000
#define GREEN		0x00ff00
#define YELLOW		0xffff00
#define CYAN		0x00ffff
#define RED			0xff0000
#define BLUE		0x0000ff
#define ORANGE 		0xFFA500


/* functions */
void LDS285_Init_Display(void);
void LDS285_Init_GPIO(void);
void LDS285_Init_FSMC(void);
void LDS285_Demo(void);

void writeCommand(uint8_t command);
void writeData(uint8_t data);

void Delay_N95(__IO uint32_t nCount);
void Delay_ms_N95(uint16_t ms);

void setBacklight(uint8_t prozent);
void fillScreen(uint32_t color);
void clearScreen(void);
void sleepDisplay(void);
void wakeDisplay(void);
void fillScreenPartial(uint32_t xstart, uint32_t xend, uint32_t ystart, uint32_t yend, uint32_t color);
void setPixel(uint32_t x, uint32_t y, uint32_t color);


#endif /* STM32F4_NOKIAN958GB_LDS285_H_ */
